/*
 *
 * Copyright (C) 2003 Sadysta ( http://sadysta.ath.cx )
 * Copyright (C) 2007 Tomasz Ostrowski ( http://ostry.w.szu.pl ) - modified from 1.8.9 source
 *
 * This file is part of Free GG.
 *
 * Free GG is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Free GG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Free GG; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

//---------------------------------------------------------------------------

#ifndef GaduThreadH
#define GaduThreadH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Scktcomp.hpp>
//---------------------------------------------------------------------------
class TGaduThread : public TThread
{
private:
    AnsiString asGaduStatus;
    void __fastcall SetStatusDesc();    
protected:
        void __fastcall Execute();
public:
        __fastcall TGaduThread(bool CreateSuspended);
        __fastcall ~TGaduThread();
        void __fastcall LogMsg();
        void __fastcall LogMessage (AnsiString S);

        void __fastcall SetStatus (AnsiString S);
        void __fastcall SetStat();

        void __fastcall LogActivity (int uin, int status, AnsiString desc);
        void __fastcall LogActiv();

        void ChangeStatus (int id, AnsiString desc, bool friends_only = false, bool force_now = false);
        void SendNotifyList();
        void ExecuteWriteQueue();

        void NotifyReply (BYTE *packet, INT size);
        void NotifyReply2 (int uin, int status, AnsiString desc, int ip = 0, int port = 0);

        void ReceiveMsg (BYTE *packet, INT size);
        void StatusUpdate (BYTE *packet, INT size);
        void SendMsg (int UIN, AnsiString msg);
        void MessageAck (BYTE *packet);
        void __fastcall ShowMsg();
        void PubdirRequest(int type, int seq, AnsiString request);
        void PubdirReply (BYTE *packet, INT size);
        int StatusToImageIndex (int status);
        void RemoveNotify (int uin, char type);
        void AddNotify (int uin, char type);
        TMenuItem* StatusToMenuItem(int status);
        void __fastcall ShowSearchResults();
        void __fastcall ShowReadResults();
        void __fastcall WriteSuccessful();

        TClientSocket *S;
        TWinSocketStream *SS;
        TStringList *WriteQueue;
        DWORD LastWrite;
        AnsiString Pubdir50_Reply;
        int err_count;
        bool dead;
};
//---------------------------------------------------------------------------
#define GG_WELCOME              0x0001
#define GG_LOGIN                0x000c
#define GG_NEW_STATUS           0x0002
#define GG_NOTIFY               0x0010
#define GG_LIST_EMPTY           0x0012
#define GG_NOTIFY_REPLY         0x000c
#define GG_ADD_NOTIFY           0x000d
#define GG_REMOVE_NOTIFY        0x000e
#define GG_STATUS               0x0002
#define GG_LOGIN_OK             0x0003
#define GG_LOGIN_FAILED         0x0009
#define GG_DISCONNECTING        0x000b
#define GG_PING                 0x0008
#define GG_PONG                 0x0007
#define GG_RECV_MSG             0x000a
#define GG_SEND_MSG             0x000b
#define GG_SEND_MSG_ACK         0x0005

#define GG_PUBDIR50_REQUEST     0x0014
#define GG_PUBDIR50_REPLY       0x000e

#define GG_PUBDIR50_WRITE         0x01
#define GG_PUBDIR50_READ          0x02
#define GG_PUBDIR50_SEARCH        0x03
#define GG_PUBDIR50_SEARCH_REPLY  0x05

#define GG_USER_BUDDY         0x01
#define GG_USER_FRIEND        0x03
#define GG_USER_BLOCKED       0x04

#define GG_CLASS_QUEUED 0x0001
#define GG_CLASS_MSG 0x0004
#define GG_CLASS_CHAT 0x0008
#define GG_CLASS_CTCP 0x0010
#define GG_CLASS_ACK 0x0020

#define GG_ACK_DELIVERED 0x0002
#define GG_ACK_QUEUED 0x0003
#define GG_ACK_MBOXFULL 0x0004
#define GG_ACK_NOT_DELIVERED 0x0006

#define GG_STATUS_NOT_AVAIL 0x0001              // Niedostpny
#define GG_STATUS_NOT_AVAIL_DESCR 0x0015        // Niedostpny (z opisem)
#define GG_STATUS_AVAIL 0x0002                  // Dostpny
#define GG_STATUS_AVAIL_DESCR 0x0004            // Dostpny (z opisem)
#define GG_STATUS_BUSY 0x0003                   // Zajty
#define GG_STATUS_BUSY_DESCR 0x0005             // Zajty (z opisem)
#define GG_STATUS_INVISIBLE 0x0014              // Niewidoczny
#define GG_STATUS_INVISIBLE_DESCR 0x0016        // Niewidoczny z opisem
#define GG_STATUS_BLOCKED 0x0006                // Zablokowany
#define GG_STATUS_FRIENDS_MASK 0x8000           // Maska bitowa oznaczajca tryb tylko dla przyjaci

#define GG_LOGIN_EXT            0x0013
#define GG_HAS_AUDIO_MASK       0x40000000

struct gg_header
{
        int type;
        int length;
};

struct gg_welcome
{
        int seed;
};

struct gg_login
{
        int uin;
        int hash;
        int status;
        int version;
        int local_ip;
        short local_port;
//        char description[];
//        int time;
};

#define GG_LOGIN_SIZE 22

struct gg_login_ext {
	int uin;		/* numer klienta */
	int hash;		/* hash hasa */
	int status;		/* pocztkowy stan */
	int version;		/* wersja klienta */
	int local_ip;		/* mj adres ip */
	short local_port;	/* port, na ktrym sucham */
	int external_ip;	/* zewntrzny adres ip */
	short external_port;	/* zewntrzny port */
//	char description[];	/* opis, nie musi wystpi */
//	int time;		/* czas, nie musi wystpi */
};

#define GG_LOGIN_EXT_SIZE 28

struct gg_new_status
{
        int status;
//        char description[];
//        int time;
};

struct gg_notify
{
        int uin;
        char type;
};

#define GG_NOTIFY_SIZE 5

struct gg_notify_reply
{
        int uin;		/* numerek */
	int status;		/* status danej osoby */
	int remote_ip;		/* adres ip delikwenta */
	short remote_port;	/* port, na ktrym sucha klient */
	int version;		/* wersja klienta */
	short unknown1;		/* znowu port? */
	// char description[];	/* opis, nie musi wystpi */
	// int time;
};

#define GG_NOTIFY_REPLY_SIZE 20

struct gg_status
{
        int uin;
        int status;
        // char description[];
        // int time;
};

struct gg_recv_msg {
	int sender;		/* numer nadawcy */
	int seq;		/* numer sekwencyjny */
	int time;		/* czas nadania */
	int cls;		/* klasa wiadomoci */
	// char message[];		/* tre wiadomoci */
};

struct gg_send_msg {
	int recipient;		/* numer odbiorcy */
	int seq;		/* numer sekwencyjny */
	int cls;		/* klasa wiadomoci */
	// char message[];		/* tre */
};

struct gg_send_msg_ack {
	int status;	/* stan wiadomoci */
	int recipient;	/* numer odbiorcy */
	int seq;	/* numer sekwencyjny */
};

struct gg_pubdir50 {
	char type;
	int seq;
	// char request[];
};

#define GG_PUBDIR50_SIZE 5

struct gg_pubdir50_reply {
	char type;
	int seq;
	// char reply[];
};

#define GG_PUBDIR50_REPLY_SIZE 5

struct gg_remove_notify {
	int uin;	/* numerek */
	char type;	/* rodzaj uytkownika */
};

#define GG_REMOVE_NOTIFY_SIZE 5

struct gg_add_notify {
	int uin;	/* numerek */
	char type;	/* rodzaj uytkownika */
};

#define GG_ADD_NOTIFY_SIZE 5

int gg_http_hash(const char *format, ...);
char *gg_urlencode(const char *str);
char *gg_urldecode (const char *str);
//---------------------------------------------------------------------------
#endif
